// 
// Copyright (c) 1984-2007 Sangoma Technologies Inc.
//
// Author	:	David Rokhvarg	<davidr@sangoma.com>
//
// sample.cpp: 
//	Sample C++ program for the following Sangoma APIs:
//	1. HDLC  Streaming API
//	2. Bit Streaming API
//	3. Frame Relay API
//	4. CHDLC API
//	5. LAPD (Q.921) API (Experimental)
//	6. "TDM VOICE API"
//
//Note 1: the header files are in "include" directory,
//		your compiler should know where to look for
//		the Header files.
//
//
// Target: Windows XP/2003 and later. Win32 Console.
//
// For Version beta-6,0,4,5 and later of the API driver (sdladrv.sys)
//

//////////////////////////////////////////////////////////////////
//Device Names formed as follows:
// "\\\\.\\WANPIPE1_IF0" - corresponds to Group 1 on WANPIPE1
//
// "\\\\.\\WANPIPE1_IF1" - corresponds to Group 2 on WANPIPE1
//
// "\\\\.\\WANPIPE1_IF2" - corresponds to Group 3 on WANPIPE1
//
// "\\\\.\\WANPIPE2_IF0" - corresponds to Group 1 on WANPIPE2
//
// "\\\\.\\WANPIPE2_IF1" - corresponds to Group 2 on WANPIPE2
//
//and so on...
//
//(WANPIPE1 is Port 1 on the first installed card)
//////////////////////////////////////////////////////////////////

#define SET_DRV_CFG 0
#if SET_DRV_CFG
#include "driver_configurator.h"
#endif

#include "sangoma_interface.h"

//globals:
unsigned int	wanpipe_number = 1;	
unsigned int	interface_number = 0;	
unsigned int	silent = 0;
unsigned int	tdm_voice_bitswap = 0;
unsigned int	txlength = 128;

static char		szTxFileName[MAX_PATH];

//mutex for synchronizing access to 'stdout' between the threads
CRITICAL_SECTION	PrintCriticalSection;
//mutex for TDM events
HANDLE	tdm_event_mutex = NULL;

callback_functions_t callback_functions;

//function prototypes:
static int got_rx_data(void *sang_if_ptr, void *rx_data);
static void got_TdmApiEvent(void *sang_if_ptr, void *event_data);

//////////////////////////////////////////////////////////////////

typedef struct{
	void			*sang_if_ptr;
	RX_DATA_STRUCT	event_data;
}TDM_API_EVENT_THREAD_PARAM;

DWORD TdmApiEventThreadFunc(LPDWORD lpdwParam);
//////////////////////////////////////////////////////////////////
#define DBG_MAIN	if(1)printf
#define ERR_MAIN	if(1)printf
#define INFO_MAIN	if(1)printf
//////////////////////////////////////////////////////////////////

sangoma_interface* init(int wanpipe_number, int interface_number)
{
	sangoma_interface	*sang_if = NULL;

	DBG_MAIN("init()\n");

	callback_functions.got_rx_data = got_rx_data;
	callback_functions.got_TdmApiEvent = got_TdmApiEvent;

	sang_if = new sangoma_interface(wanpipe_number, interface_number);
	if(sang_if->init(&callback_functions)){
		delete sang_if;
		return NULL;
	}

	DBG_MAIN("init(): OK\n");
	return sang_if;
}

void cleanup(sangoma_interface	*sang_if)
{
	DBG_MAIN("cleanup()\n");

	if(sang_if){
		delete sang_if;
	}
}

int start(sangoma_interface	*sang_if)
{
	DBG_MAIN("start()\n");
	return sang_if->run();
}

void stop(sangoma_interface	*sang_if)
{
	DBG_MAIN("stop()\n");
	sang_if->stop();
}

void PrintRxData(RX_DATA_STRUCT* pRx)
{
	api_header_t*	pri;
	USHORT			datlen;
	PUCHAR			data;
	static	rx_counter = 0;

	pri = &pRx->api_header;

	//NOTE: if running in BitStream mode, there will be TOO MUCH to print 
	datlen = pri->data_length;
	data =  pRx->data;

	rx_counter++;
	if(silent){
		if((rx_counter % 100) == 0){
			printf("Rx counter:%d, Rx datlen : %d. Data :\n", rx_counter, datlen);
		}
		return;
	}else{
		printf("Rx counter:%d, Rx datlen : %d. Data :\n", rx_counter, datlen);
	}

#if 1
	for(int ln = 0; ln < datlen; ln++){
		if((ln % 20 == 0)){
			if(ln){
				printf("\n");
			}
			printf("%04d ", ln/20);
		}
		printf("%02X ", data[ln]);
	}
	printf("\n");
#endif
}

static int got_rx_data(void *sang_if_ptr, void *rx_data)
{
	sangoma_interface	*sang_if;
	//Do something with data received from Sangoma interface.
	//
	//Fore example, transimit back everything what was received:
	sang_if = (sangoma_interface*)sang_if_ptr;

	if(tdm_voice_bitswap){
		sang_if->bit_swap_a_buffer(((TX_RX_DATA_STRUCT*)rx_data)->data,
			((TX_RX_DATA_STRUCT*)rx_data)->api_header.data_length);
	}
	//sang_if->transmit((TX_RX_DATA_STRUCT*)rx_data);

	EnterCriticalSection(&PrintCriticalSection);
	PrintRxData((RX_DATA_STRUCT*)rx_data);
	LeaveCriticalSection(&PrintCriticalSection);

	return 0;
}

static void got_TdmApiEvent(void *sang_if_ptr, void *event_data)
{
	TDM_API_EVENT_THREAD_PARAM	*param;
	DWORD						dwThreadId;

	WaitForSingleObject(tdm_event_mutex, 1000);

	param = (TDM_API_EVENT_THREAD_PARAM*)malloc(sizeof(TDM_API_EVENT_THREAD_PARAM));
	if(param == NULL){
		ERR_MAIN("Failed to allocate memory for 'Event Thread parameter'!!\n");
		ReleaseMutex(tdm_event_mutex);
		return;
	}

	memcpy(&param->event_data, event_data, sizeof(RX_DATA_STRUCT));
	param->sang_if_ptr = sang_if_ptr;

	//
	//Handling of Events must be done OUTSIDE of the REAL-TIME Rx thread.
	//
	//Create a special thread for Event hadling.
	//
	if(CreateThread( 
        NULL,                       /* no security attributes        */ 
        0,                          /* use default stack size        */ 
        (LPTHREAD_START_ROUTINE)TdmApiEventThreadFunc, /* thread function     */ 
        param,						/* argument to thread function   */ 
        0,                          /* use default creation flags    */ 
        &dwThreadId					/* returns the thread identifier */ 
		) == NULL){
		ERR_MAIN("Failed to create 'TdmApiEvent' thread!!\n");
	}
	ReleaseMutex(tdm_event_mutex);
}

DWORD TdmApiEventThreadFunc(LPDWORD lpdwParam)
{ 
	TDM_API_EVENT_THREAD_PARAM	*param;
	RX_DATA_STRUCT				*pEvent;
	sangoma_interface			*sang_if;
	wp_tdm_api_event_t			*wp_tdm_api_event;

	WaitForSingleObject(tdm_event_mutex, 1000);

	param = (TDM_API_EVENT_THREAD_PARAM*)lpdwParam;

	pEvent = &param->event_data;
	sang_if = (sangoma_interface*)param->sang_if_ptr;

	wp_tdm_api_event = (wp_tdm_api_event_t*)pEvent->data;


	DBG_MAIN( "TdmApiEventThreadFunc:: ifname: %s\n", sang_if->device_name);

	switch(wp_tdm_api_event->wp_tdm_api_event_type)
	{
	case WP_TDMAPI_EVENT_DTMF:
		DBG_MAIN("DTMF Event: Channel: %d, Digit: %c (Port: %s, Type:%s)!\n",
			wp_tdm_api_event->channel,
			wp_tdm_api_event->wp_tdm_api_event_dtmf_digit,
			(wp_tdm_api_event->wp_tdm_api_event_dtmf_port == WAN_EC_CHANNEL_PORT_ROUT)?"ROUT":"SOUT",
			(wp_tdm_api_event->wp_tdm_api_event_dtmf_type == WAN_EC_TONE_PRESENT)?"PRESENT":"STOP");		

		break;

	case WP_TDMAPI_EVENT_RXHOOK:
		DBG_MAIN("RXHOOK Event: Channel: %d, %s! (0x%X)\n", 
			wp_tdm_api_event->channel,
			WAN_EVENT_RXHOOK_DECODE(wp_tdm_api_event->wp_tdm_api_event_hook_state),
			wp_tdm_api_event->wp_tdm_api_event_hook_state);

		sang_if->last_hook_state_event = wp_tdm_api_event->wp_tdm_api_event_hook_state;

		break;

	case WP_TDMAPI_EVENT_RING_DETECT:
		DBG_MAIN("RING Event: %s! (0x%X)\n",
			WAN_EVENT_RING_DECODE(wp_tdm_api_event->wp_tdm_api_event_ring_state),
			wp_tdm_api_event->wp_tdm_api_event_ring_state);
		break;

	case WP_TDMAPI_EVENT_RING_TRIP_DETECT:
		DBG_MAIN("RING TRIP Event: %s! (0x%X)\n", 
			WAN_EVENT_RING_TRIP_DECODE(wp_tdm_api_event->wp_tdm_api_event_ring_state),
			wp_tdm_api_event->wp_tdm_api_event_ring_state);
		break;

	case WP_TDMAPI_EVENT_RBS:
		DBG_MAIN("RBS Event: Channel: %d, 0x%X!\n",
			wp_tdm_api_event->channel,
			wp_tdm_api_event->wp_tdm_api_event_rbs_bits);		
		DBG_MAIN( "RX RBS: A:%1d B:%1d C:%1d D:%1d\n",
			(wp_tdm_api_event->wp_tdm_api_event_rbs_bits & WAN_RBS_SIG_A) ? 1 : 0,
			(wp_tdm_api_event->wp_tdm_api_event_rbs_bits & WAN_RBS_SIG_B) ? 1 : 0,
			(wp_tdm_api_event->wp_tdm_api_event_rbs_bits & WAN_RBS_SIG_C) ? 1 : 0,
			(wp_tdm_api_event->wp_tdm_api_event_rbs_bits & WAN_RBS_SIG_D) ? 1 : 0);
		break;

	default:
		ERR_MAIN("Unknown TDM API Event: %d\n", wp_tdm_api_event->wp_tdm_api_event_type);
		break;
	}

	free(lpdwParam);
	ReleaseMutex(tdm_event_mutex);

	//Done with the Event, exit the thread.
	return 0;
}

int tx_file(sangoma_interface *sang_if)
{
	FILE			*pFile;
	unsigned int	tx_counter=0, bytes_read_from_file, total_bytes_read_from_file=0;
	TX_DATA_STRUCT	local_tx_data;

	pFile = fopen( szTxFileName, "rb" );
	if( pFile == NULL){
		printf( "Can't open file: [%s]\n", szTxFileName );
		return 1;
	}

	do
	{
		//read tx data from the file. if 'bytes_read_from_file != TX_LENGTH', end of file is reached
		bytes_read_from_file = fread( local_tx_data.data, 1, txlength /* MTU size */, pFile );
		total_bytes_read_from_file += bytes_read_from_file;

		local_tx_data.api_header.data_length = txlength;//ALWAYS transmit MTU size over the bitstream
		local_tx_data.api_header.operation_status = SANG_STATUS_TX_TIMEOUT;
		
		sang_if->transmit(&local_tx_data);
		
		tx_counter++;
		
		//printf("tx_counter: %u\r",tx_counter);

	}while((int)bytes_read_from_file == txlength);

	printf("Finished transmitting file \"%s\" (tx_counter: %u, total_bytes_read_from_file: %d)\n",
		szTxFileName, tx_counter, total_bytes_read_from_file);
	fclose( pFile );

	return 0;
}

static int get_user_number()
{
	int result = 1;
	int retry_counter = 0;

	while(scanf("%d", &result) == 0){
		fflush( stdin );
		printf("\nError: Not a numerical input!! Must be from 1 to 24.\n");
		if(retry_counter++ > 10){
			printf("giving up...\n");
			result = 1;
			break;
		}
	}//while()

	printf("User input: %d\n", result);
	return result;
}

static int parse_command_line_args(int argc, char* argv[])
{
	int i;
#define USAGE_STR \
"\n\
Usage: sample [-c] [-i] [-silent]\n\
\n\
Options:\n\
\t-c		number	Wanpipe number: 1,2,3...\n\
\t-i		number	Interface number 0,1,2,3,....\n\
\t-silent			Disable display of Rx data\n\
\t-tdmvbitswap		To enable BitSwap of voice data\n\
\n\
Example: sample -c 1 -i 0\n"

	for(i = 0; i < argc; i++){
	
		if(strcmp(argv[i], "-silent") == 0){
			printf("disabling Rx data display...\n");
			silent = 1;
		}else if(strcmp(argv[i], "help") == 0 || strcmp(argv[i], "?") == 0 || strcmp(argv[i], "/?") == 0){
			printf(USAGE_STR);
			return 1;
		}else if(strcmp(argv[i], "-c") == 0){
			if (i+1 > argc-1){
				printf("No Wanpipe number was provided!\n");
				return 1;
			}
			wanpipe_number = atoi(argv[i+1]);
			printf("Using wanpipe number %d\n", wanpipe_number);
		}else if(strcmp(argv[i], "-i") == 0){
			if (i+1 > argc-1){
				printf("No Interface number was provided!\n");
				return 1;
			}
			interface_number = atoi(argv[i+1]);
			printf("Using interface number %d\n", interface_number);
		}else if(strcmp(argv[i], "-tdmvbitswap") == 0){
			printf("enabling TDM Voice BitSwap...\n");
			tdm_voice_bitswap = 1;
		}
	}
	return 0;
}

#if !SET_DRV_CFG
int main(int argc, char* argv[])
{
	char				rc;
	HANDLE				current_process = NULL;
	char				user_selection;
	sangoma_interface	*sang_if = NULL;
	TX_DATA_STRUCT		local_tx_data;
	UCHAR				tx_test_byte = 0;

	if(parse_command_line_args(argc, argv)){
		return 1;
	}

	////////////////////////////////////////////////////////////////////////////
	//initialize mutex objects
	InitializeCriticalSection(&PrintCriticalSection);

	tdm_event_mutex = CreateMutex(NULL, FALSE, NULL);
	if(tdm_event_mutex == NULL){
		printf("Failed to create tdm_event_mutex!!!!\n");
		return 1;
	}

	////////////////////////////////////////////////////////////////////////////
	current_process = GetCurrentProcess();
	//this program 'talks' direcly to hardware, it has to be real time class.
	if(SetPriorityClass(current_process, REALTIME_PRIORITY_CLASS) == 0){
		ERR_MAIN("Failed to set program priority!\n!");
		return 1;
	}

	////////////////////////////////////////////////////////////////////////////
	//User may provide Wanpipe Number and Interface Number as a command line arguments:
	printf("Using wanpipe_number: %d, interface_number: %d\n", 
		wanpipe_number, interface_number);

	sang_if = init(wanpipe_number, interface_number);

	if(sang_if == NULL){
		return 1;
	}

	rc = start(sang_if);
	if(rc){
		cleanup(sang_if);
		return rc;
	}

	do{
		EnterCriticalSection(&PrintCriticalSection);
		INFO_MAIN("Press 'q' to quit the program.\n");
		INFO_MAIN("Press 't' to transmit data.\n");
		INFO_MAIN("Press 's' to get Operational Statistics.\n");
		INFO_MAIN("Press 'f' to reset (flush) Operational Statistics.\n");

		if(sang_if->get_adapter_type() == WAN_MEDIA_T1 || sang_if->get_adapter_type() == WAN_MEDIA_E1){
			printf("Press 'a' to get T1/E1 alarms.\n");
			//RBS (CAS) commands
			printf("Press 'g' to get RBS bits.\n");
			printf("Press 'r' to set RBS bits.\n");
		}
		printf("Press 'i' to set Tx idle data buffer (BitStream only).\n");
		switch(sang_if->get_adapter_type())
		{
		case WAN_MEDIA_T1:
			//those commands valid only for T1
			INFO_MAIN("Press 'l' to send 'activate remote loop back' signal.\n");
			INFO_MAIN("Press 'd' to send 'deactivate remote loop back' signal.\n");
			break;

		case WAN_MEDIA_FXOFXS:
			switch(sang_if->get_sub_media())
			{
			case MOD_TYPE_FXS:
				INFO_MAIN("Press 'e' to listen to test tones on a phone connected to the A200-FXS\n");
				INFO_MAIN("Press 'c' to ring/stop ring phone connected to the A200-FXS\n");
				INFO_MAIN("Press 'n' to enable/disable reception of ON/OFF Hook events on A200-FXS\n");
				INFO_MAIN("Press 'm' to enable DTMF events (on SLIC chip) on A200-FXS\n");
				INFO_MAIN("Press 'j' to enable/disable reception of Ring Trip events on A200-FXS\n");
				break;

			case MOD_TYPE_FXO:
				INFO_MAIN("Press 'u' to enable/disable reception of Ring Detect events on A200-FXO\n");
				INFO_MAIN("Press 'h' to transmit ON/OFF hook signals on A200-FXO\n");
				break;
			}
			break;
		}
		INFO_MAIN("Press 'o' to enable DTMF events (on Octasic chip)\n");
		LeaveCriticalSection(&PrintCriticalSection);

		user_selection = tolower(getch());
		switch(user_selection)
		{
		case 'q':
			break;
		case 't':
			int cnt;
			for(cnt = 0; cnt < 1; cnt++){
				local_tx_data.api_header.data_length = txlength;
				local_tx_data.api_header.operation_status = SANG_STATUS_TX_TIMEOUT;

				//set the actual data for transmission
				memset(local_tx_data.data, tx_test_byte, txlength);

				sang_if->transmit(&local_tx_data);

				tx_test_byte++;
			}
			break;

		case 's':
			sang_if->operational_stats();
			break;

		case 'f':
			sang_if->flush_operational_stats();
			break;


		case 'v':
			//read API driver version
			sang_if->read_api_driver_version();
			break;

		case 'a':
			//read T1/E1/56k alarms
			sang_if->read_te1_56k_stat();
			break;

		case 'l'://send activate remote loop back signal
			sang_if->set_lb_modes(WAN_TE1_TX_LB_MODE, WAN_TE1_ACTIVATE_LB);
			break;

		case 'd'://send deactivate remote loop back signal
			sang_if->set_lb_modes(WAN_TE1_TX_LB_MODE, WAN_TE1_DEACTIVATE_LB);
			break;

		case 'g'://read RBS bits
			{
				rbs_management_t rbs_management_struct = {0,0};

				sang_if->enable_rbs_monitoring();

				printf("Type Channel number and press <Enter>:\n");
				rbs_management_struct.channel = get_user_number();//channels (Time Slots). Valid values: 1 to 24.

				//IMPORTANT: this call must be thread safe - use mutex or critical section around it.
				sang_if->get_rbs(&rbs_management_struct);
			}
			break;

		case 'r'://set RBS bits
			{
				rbs_management_t rbs_management_struct = {0,0};

				sang_if->enable_rbs_monitoring();

				printf("Type Channel number and press <Enter>:\n");
				rbs_management_struct.channel = get_user_number();//channels (Time Slots). Valid values: 1 to 24.

				/*	bitmap - set as needed: WAN_RBS_SIG_A |	WAN_RBS_SIG_B | WAN_RBS_SIG_C | WAN_RBS_SIG_D;

					In this example make bits A and B to change each time,
					so it's easy to see the change on the receiving side.
				*/
				if(rbs_management_struct.ABCD_bits == WAN_RBS_SIG_A){
					rbs_management_struct.ABCD_bits = WAN_RBS_SIG_B;
				}else{
					rbs_management_struct.ABCD_bits = WAN_RBS_SIG_A;
				}

				//IMPORTANT: this call must be thread safe - use mutex or critical section around it.
				sang_if->set_rbs(&rbs_management_struct);
			}
			break;

		case 'i':
			sang_if->set_idle_tx_data_buffer(0x00);
			break;

		case 'c':
			printf("Press 'e' to START ring, 'd' to STOP ring.\n");
			printf("\n");
			switch(tolower(getch()))
			{
			case 'e':
				sang_if->start_ringing_phone();//start
				break;
			case 'd':
			default:
				sang_if->stop_ringing_phone();//stop
			}
			break;

		case 'e':
			printf("Press 'e' to START a Tone, 'd' to STOP a Tone.\n");
			printf("\n");

			switch(tolower(getch()))
			{
			case 'e':
				printf("Press 'r' for Ring Tone, 'd' for Dial Tone, 'b' for Busy Tone, 'c' for Congestion Tone.\n");
				printf("\n");
				switch(tolower(getch()))
				{
				case 'r':
					sang_if->start_ring_tone();
					break;
				case 'd':
					sang_if->start_dial_tone();
					break;
				case 'b':
					sang_if->start_busy_tone();
					break;
				case 'c':
				default:
					sang_if->start_congestion_tone();
					break;
				}
				break;

			case 'd':
			default:
				sang_if->stop_all_tones();//stop all tones
			}
			break;

		case 'n':
			printf("Press 'e' to ENABLE Rx Hook Events, 'd' to DISABLE Rx Hook Events.\n");
			printf("\n");
			switch(tolower(getch()))
			{
			case 'e':
				sang_if->sangoma_tdm_enable_rxhook_events();
				break;
			case 'd':
			default:
				sang_if->sangoma_tdm_disable_rxhook_events();
			}
			break;

		case 'm':
			//Enable/Disable DTMF events on SLIC chip.
			//On Analog (A200) card only.
			printf("Press 'e' to ENABLE Remora DTMF Events, 'd' to DISABLE Remora DTMF Events.\n");
			printf("\n");
			switch(tolower(getch()))
			{
			case 'e':
				sang_if->sangoma_tdm_enable_rm_dtmf_events();
				break;
			case 'd':
			default:
				sang_if->sangoma_tdm_disable_rm_dtmf_events();
			}
			break;

		case 'o':
			{
				//Enable DTMF events on Octasic chip. 
				//For both Analog (A200) and T1/E1 (A104D) cards, but only if the chip is present.
				printf("Press 'e' to ENABLE Octasic DTMF Events, 'd' to DISABLE Octasic DTMF Events.\n");
				int channel;

				printf("\n");
				switch(tolower(getch()))
				{
				case 'e':
					printf("Type Channel number and press <Enter>:\n");
					channel = get_user_number();//channels (Time Slots). Valid values: 1 to 31.

					sang_if->sangoma_tdm_enable_dtmf_events(channel);
					break;
				case 'd':
				default:
					printf("Type Channel number and press <Enter>:\n");
					channel = get_user_number();//channels (Time Slots). Valid values: 1 to 31.

					sang_if->sangoma_tdm_disable_dtmf_events(channel);
				}
			}
			break;

		case 'u':
			//Enable/Disable Ring Detect events on FXO. 
			printf("Press 'e' to ENABLE Rx Ring Detect Events, 'd' to DISABLE Rx Ring Detect Events.\n");
			printf("\n");
			switch(tolower(getch()))
			{
			case 'e':
				sang_if->sangoma_tdm_enable_ring_detect_events();
				break;
			case 'd':
			default:
				sang_if->sangoma_tdm_disable_ring_detect_events();
			}
			break;

		case 'j':
			//Enable/Disable Ring Trip events on FXS. 
			printf("Press 'e' to ENABLE Rx Ring Trip Events, 'd' to DISABLE Rx Ring Trip Events.\n");
			printf("\n");
			switch(tolower(getch()))
			{
			case 'e':
				sang_if->sangoma_tdm_enable_ring_trip_detect_events();
				break;
			case 'd':
			default:
				sang_if->sangoma_tdm_disable_ring_trip_detect_events();
			}
			break;

		case 'h':
			printf("Press 'e' to transmit OFF hook signal, 'd' to transmit ON hook signal.\n");
			printf("\n");
			switch(tolower(getch()))
			{
			case 'e':
				sang_if->fxo_go_off_hook();
				break;
			case 'd':
			default:
				sang_if->fxo_go_on_hook();
			}
			break;

		default:
			INFO_MAIN("Invalid command.\n");
		}

	}while(user_selection != 'q');

	stop(sang_if);
	cleanup(sang_if);

	return 0;
}

#else

int main(int argc, char* argv[])
{
	int					rc = 0;
	char				user_selection;
	driver_configurator	*drv_cfg_obj;
	sdla_fe_cfg_t		sdla_fe_cfg;

	if(parse_command_line_args(argc, argv)){
		return 1;
	}
		
	drv_cfg_obj = new driver_configurator();
	if(drv_cfg_obj == NULL || drv_cfg_obj->init(wanpipe_number)){
		return 2;
	}

	do{
		INFO_MAIN("\nPress 't' to set T1 configration.\n");
		INFO_MAIN("Press 'e' to set E1 configration.\n");
		INFO_MAIN("Press 'q' to quit the program.\n");

		user_selection = tolower(getch());
		switch(user_selection)
		{
		case 'q':
			break;
		case 't'://T1
			memset(&sdla_fe_cfg, 0x00, sizeof(sdla_fe_cfg_t));

			FE_MEDIA(&sdla_fe_cfg) = WAN_MEDIA_T1;
			FE_LCODE(&sdla_fe_cfg) = WAN_LCODE_B8ZS;
			FE_FRAME(&sdla_fe_cfg) = WAN_FR_ESF;

			FE_CLK(&sdla_fe_cfg)		= WAN_NORMAL_CLK;
			//FE_REFCLK(&sdla_fe_cfg)		= 0;	//optional
			FE_HIMPEDANCE_MODE(&sdla_fe_cfg) = WANOPT_NO;
			
			FE_SIG_MODE(&sdla_fe_cfg) = WAN_TE1_SIG_CCS;

			FE_LBO(&sdla_fe_cfg) = WAN_T1_LBO_0_DB;
			break;
		case 'e':
			memset(&sdla_fe_cfg, 0x00, sizeof(sdla_fe_cfg_t));

			FE_MEDIA(&sdla_fe_cfg) = WAN_MEDIA_E1;
			FE_LCODE(&sdla_fe_cfg) = WAN_LCODE_HDB3;
			FE_FRAME(&sdla_fe_cfg) = WAN_FR_CRC4;

			FE_CLK(&sdla_fe_cfg)				= WAN_NORMAL_CLK;
			//FE_REFCLK(&sdla_fe_cfg)			= 0;//optional
			FE_HIMPEDANCE_MODE(&sdla_fe_cfg) = WANOPT_NO;
			
			FE_SIG_MODE(&sdla_fe_cfg) = WAN_TE1_SIG_CCS;
			break;
		default:
			INFO_MAIN("Invalid command.\n");
			break;
		}

		if(user_selection == 't' || user_selection == 'e'){
			drv_cfg_obj->set_t1_e1_configuration(&sdla_fe_cfg);
		}

	}while(user_selection != 'q');

	if(drv_cfg_obj != NULL)	{delete drv_cfg_obj;}

	return rc;
}

#endif // SET_DRV_CFG

